
/* Host toolkit */

/*
 * Based on:
 *
 * A.M. van den Bos' PlanMap header files
 * Micheal Keane's ALTSPEC.H
 *
 * Modification history:
 *
 * RBW 98.1.26 Incorporated CC's changes to live version
 */

#ifndef _tkhost_h
#define _tkhost_h

#include <limits.h>
#include <xstring.h>


#if INT_MIN < -2147483647 && INT_MAX >= 2147483647

typedef int            SB4;
typedef unsigned int   UB4;

#else

typedef long           SB4;
typedef unsigned long  UB4

#endif

#if SHRT_MIN < -32767 && SHRT_MAX >= 32767

typedef short          SB2;
typedef unsigned short UB2;

#else

typedef int            SB2;
typedef unsigned int   UB2

#endif

typedef signed char    SB1;
typedef unsigned char  UB1;

#define NUM_PLANETS        500   // PLANETS has a fixed number of planets
#define PLANETNAME_LENGTH  20    // As in file PLANET.NM
#define NUM_RACES          11    // There are 11 races
#define MAXSHIP            500   // Maximum number of ships
#define MAXHULL            120   // Maximum number of hulls that can be read
#define MAXBEAM            10    // Beam specs
#define MAXTORP            10    // Torp specs
#define MAXENG             9     // Engine specs
#define MAXMESSAGESIZE  (41*15)  // 15 lines of 40 characters + CR
#define MAXMESSAGE         50    // Max. number of outgoing messages

/* Length of race names (long form, plural form, adjective form) */
#define RACE_LENGTH(j) (((j)==0)?30:(((j)==1)?20:12))

#define NO_HULL_STR "CANNOT SCAN HULL              "

/**********************************************************************/
/* Maximum number of structures on a planet as function of the number */
/* of clans. (function-like macro; there must be at least 1 clan)     */
/**********************************************************************/

#define MAX_MINES(clans) ( ((clans)<=200) ? (clans) : 200+(UB2)(0.5+sqrt((clans)-200)))
#define MAX_FACT(clans)  ( ((clans)<=100) ? (clans) : 100+(UB2)(0.5+sqrt((clans)-100)))
#define MAX_DEF(clans)   ( ((clans)<= 50) ? (clans) :  50+(UB2)(0.5+sqrt((clans)- 50)))

/**********************************************************************/
/* Crew kill/damage/shields vs Beam/torp kill and mass                */
/**********************************************************************/

#define SQR(x) ((x)*(x))  // rise to the power of two
#define CREW_KILL(kill,mass)     (min(9999L,(UB4)((kill)*(80.0/((mass)+1)))))
#define SHIELD_DAMAGE(expl,mass) (min(100L,(UB4)((expl)*(80.0/((mass)+1))+1)))
#define SHIP_DAMAGE(expl,mass)   (min(100L,(UB4)((expl)*SQR(80.0/((mass)+1)))+2))

/**************************************************************************/
/* Colonists/natives status (build-screen VGA Planets)                    */
/**************************************************************************/

#define COL_STATUS(clans,tax,planet,mines,factories)        \
((int) (1000.0-sqrt((float)clans) -80*(tax) -               \
3*abs((int)50-(planet)) -((float)(mines)+(factories))/3))

#define NAV_STATUS(clans,gov,tax,planet,mines,factories)    \
((int) (1000.0-sqrt((float)clans) -85*(tax) -               \
50*((int)10-(gov)) -((float)(mines)+(factories))/2))

#define TEXT_STATUS(status)                      \
((status >=  450) ? "Love you":                  \
 (status >=   50) ? "Like your leadership":      \
 (status >= -100) ? "Are undecided about you":   \
 (status >= -550) ? "Are angry about you":       \
                    "Hate you!" )

#define SHORT_STATUS(status)     \
((status >=  450) ? "Love":      \
 (status >=   50) ? "Like":      \
 (status >= -100) ? "Neutral":   \
 (status >= -550) ? "Angry":     \
                    "Hate!")

/**************************************************************************/
/* Mineral and cargo amounts                                              */
/**************************************************************************/

struct Minerals
{
  enum Type
  {
    FUEL,
    MONEY,
    TRI,
    DUR,
    MOL,
    SUPPLIES,
    CLANS,
    CARGO_ITEMS
  };

  UB4  tri;
  UB4  dur;
  UB4  mol;
  static UB4 nothing;

  virtual UB4 &operator[](size_t type)
  {
    switch (type)
    {
      case TRI:         return tri;
      case DUR:         return dur;
      case MOL:         return mol;
      default:          nothing = 0;
                        return nothing;
    }
  }

  virtual UB4 Mass()
  {
    return tri + dur + mol;
  }
};

struct BuildAmount : public Minerals
{
  UB4  money;

  virtual UB4 &operator[](size_t type)
  {
    switch (type)
    {
      case MONEY:       return money;
      case TRI:         return tri;
      case DUR:         return dur;
      case MOL:         return mol;
      default:          nothing = 0;
                        return nothing;
    }
  }

  virtual UB4 Mass()
  {
    return tri + dur + mol;
  }
};

struct Resources : public Minerals
{
  UB4  fuel;

  virtual UB4 &operator[](size_t type)
  {
    switch (type)
    {
      case FUEL:        return fuel;
      case TRI:         return tri;
      case DUR:         return dur;
      case MOL:         return mol;
      default:          nothing = 0;
                        return nothing;
    }
  }

  virtual UB4 Mass()
  {
    return tri + dur + mol + fuel;
  }
};

struct Cargo : public Minerals
{
  UB4  fuel;
  UB4  money;
  UB4  supplies;
  UB4  clans;

  virtual UB4 &operator[](size_t type)
  {
    switch (type)
    {
      case FUEL:        return fuel;
      case MONEY:       return money;
      case TRI:         return tri;
      case DUR:         return dur;
      case MOL:         return mol;
      case SUPPLIES:    return supplies;
      case CLANS:       return clans;
      default:          nothing = 0;
                        return nothing;
    }
  }

  virtual UB4 Mass()
  {
    return tri + dur + mol + supplies + clans;
  }
};


/**************************************************************************/
/* Coordinates                                                            */
/**************************************************************************/

struct Coord
{
  UB2  x;
  UB2  y;

  Coord() {}

  Coord(UB2 xx, UB2 yy)
  {
    x = xx;
    y = yy;
  }

  Coord& operator += (const Coord &coord)
  {
    x += coord.x;
    y += coord.y;
    return *this;
  }

  Coord& operator -= (const Coord &coord)
  {
    x -= coord.x;
    y -= coord.y;
    return *this;
  }

  Coord operator+ (const Coord &coord)
  {
    return Coord(x+coord.x, y+coord.y);
  }

  Coord operator- (const Coord &coord)
  {
    return Coord(x-coord.x, y-coord.y);
  }

  UB4 operator* (const Coord &coord) // square of distance
  {
    Coord temp;
    temp=*this-coord;
    return temp.x*temp.x+temp.y*temp.y;
  }

  bool operator== (const Coord &coord)
  {
    return (coord.x == x && coord.y == y);
  }

  bool operator!= (const Coord &coord)
  {
    return (coord.x != x || coord.y != y);
  }

};



/**************************************************************************/
/* RACE.NM file                                                           */
/**************************************************************************/

typedef struct
{ XString sFull;
  XString sPlural;
  XString sAdjective;
} RACENM;

/**************************************************************************/
/* XYPLAN.DAT file with (x,y) coordinates of the 500 planets.             */
/* The file is 3000 bytes and can be loaded directly into a               */
/* PLANET_COORDS[NUM_PLANETS] stucture. The dummy field is not used and   */
/* always (?) zero. The 1-based index is the ID of the planet.            */
/*                                                                        */
/* PLANET.NM file with the 500 planet names. 10000 bytes in size and can  */
/* be loaded directly into a CHAR[NUM_PLANETS][20] structure. A 1-based   */
/* index is the ID of the planet.                                         */
/**************************************************************************/

typedef struct
{  Coord pos;
   UB2  owner; // in .hst file, anyway
} PLANET_COORDS;

typedef struct
{  XString sName; // [20]
} PLANET_NAME;

/***************************************************************************/
/* Hull, engine, beam & torpedoes specifications.                          */
/* VGA Planets calculates a single UB4  checksum over the specification    */
/* files HULLSPEC.DAT, ENGSPEC.DAT, BEAMSPEC.DAT & TORPSPEC.DAT by summing */
/* the BYTES of each file.                                                 */
/***************************************************************************/

typedef struct
{ UB2 wHullnum;
} TRUEHULL;

typedef struct              // HULLSPEC.DAT
{ XString sName;            // Name of the hull [30]
  UB2  wPicture;            // Picture ID
  UB2  wDamagePic;          // Always 1, perhaps part of Picture ID; I don't know
  BuildAmount cargo;        // Amount of resources to build
  UB2  wFuel, wCrew;        // Size of fuel tanks & crew
  UB2  wEngines, wMass;     // Engines needs & Mass
  UB2  wTechLevel, wCargo;  // Tech level needs and max cargo
  UB2  wFighterBays;        // Fighter bays
  UB2  wTorpsBays;          // Torp bays
  UB2  wBeams;              // Max. number of beams
} HULLSPEC;

typedef struct              // BEAMSPEC.DAT
{ XString sName;            // Name of beam weapon [20]
  BuildAmount cargo;        // cost to build beam
  UB2  wMass;               // Total mass of beam weapon
  UB2  wTechLevel;          // Tech level required to build
  UB2  wKill;               // Kill level of weapon
  UB2  wExplosive;          // Destructive level of weapon
} BEAMSPEC;

typedef struct              // TORPSPEC.DAT
{ XString sName;            // Name of torp. launcher [20]
  UB2  wCostTorp;           // Costs of one torpedo of this type
  BuildAmount cargo;        // cost to build launcher
  UB2  wMass;               // Total mass of torp. launcher
  UB2  wTechLevel;          // Tech level required to build torp. launcher
  UB2  wKill;               // Kill level of weapon
  UB2  wExplosive;          // Destructive level of weapon
} TORPSPEC;

typedef struct              // ENGSPEC.DAT
{ XString sName;            // Name of engine [20]
  BuildAmount cargo;        // cost to build it
  UB2  wTechLevel;          // Tech level required to build it.
  UB4  lFuelUse[10];        // Warp x requires lFuelUse[x]
} ENGSPEC;

/***************************************************************************/
/* SHIP??.DAT First UB2  of file #ships, followed by a list of ships.      */
/* SHIP.HST   First UB2  is a dummy, followed by an ARRAY[1..500] OF       */
/*            SHIPDATA. The index is the ship's ID                         */
/*************  **************************************************************/

typedef struct
{ UB2  wID;                 // ship's ID
  UB2  wRace;               // owner of ship  (0=>SHIP.HST => no ship)
                            // 12 = "unknown race"
  XString sCode;            // Friendly code; three characters [3]
  UB2  wWarp;               // Speed of ship
  Coord dis;                // Displacement next turn (tan(bearing) = xDis/yDis)
  Coord pos;                // Location of ship (xCoord, yCoord)
  UB2  wDriveLevel;         // Tech level of engines
  UB2  wShipType;           // Type of ship (index in HULLSPEC.DAT)
  UB2  wBeamLevel;          // Tech level of beams
  UB2  wBeamWeapons;        // Number of beam weapons
  UB2  wFighterBays;        // Number of fighter bays
  UB2  wTorpLevel;          // Torp. tech. level
  UB2  wTorps;              // Number of torps/fighters
  UB2  wTorpsBays;          // Number of torp. lauchers
  UB2  wMission;            // Mission of the ship
  UB2  wEnemy;              // Primairy enemy
  UB2  wTowed;              // Ship being towed
  UB2  wDamage;             // Ships damage in %
  UB2  wCrew;               // Number of crew members on ship
  XString  sName;           // Name of ship [20]
  Cargo cargo;              // combined cargo structure
  Cargo toplanet;
  UB2 wPID;
  Cargo toship;
  UB2 wSID;

  UB2  wIntercept;          // ID of ship to intercept
  HULLSPEC hullspec;
  ENGSPEC engspec;
  BEAMSPEC beamspec;
  TORPSPEC torpspec;
} SHIPDATA;

/**************************************************************************/
/* TARGET??.DAT file with scanned enemy hulls.                            */
/**************************************************************************/

typedef struct
{ UB2  wID;                // ID of ship
  UB2  wRace;              // Owner
  UB2  wWarp;              // Current speed
  Coord pos;               // Location
  UB2  wHull;              // Index in HULLSPEC.DAT
  UB2  wBearing;           // In degrees
  XString sName;           // Name of ship [20]
} TARGETDATA;

/***********************************************************************/
/* SHIPXY??.DAT & SHIPXY.HST. File with (scanned) enemy ships          */
/* 500 entries, position in file is ship's ID                          */
/***********************************************************************/

typedef struct
{ Coord pos;                // Location of ship
  UB2  wOwner;              // wOwner = 0 -> no ship scanned with this ID
  UB2  wMass;               // Mass of ship
} SHIPXY;

/*************************************************************************/
/* PDATA??.DAT & PDATA.HST first word in file equals #planets or 0 resp. */
/*************************************************************************/

typedef struct
{ Coord pos;                // brought in from elsewhere
  XString sName;            // brought in from elsewhere
  UB2  wOwner;              // Owner (0 = no owner)
  UB2  wID;                 // ID of planet
  XString sCode;            // Friendly code [3]
  UB2  wMines;              // Number of mineral mines
  UB2  wFactories;          // Number of factories
  UB2  wDefense;            // Defences on planet

  Cargo cargo;              // combined cargo structure
  Resources left;           // minerals left in planet
  Resources density;        // density of minerals

  UB2  wColTax;             // Colonist-taxes in %
  UB2  wNativeTax;          // Native-taxes in %
  SB2  iColStatus;          // <20 fighting, 20 rioting, 40 very angry, 50 unhappy, 70 calm, 90 happy
  SB2  iNativeStatus;       // id.

  UB2  wGov;                // Government of natives (6=monarcy 5=feudel etc.)
  UB4  lNatives;            // Number of natives
  UB2  wRace;               // Native's race (7=amph 8=ghip etc.)

  UB2  wTemperature;        // Climate. Defines planet picture & color-palette for picture (?)
  UB2  BuildNewBase;        // Next turn, a base will be here
} PLANETDATA;

/**************************************************************************/
/* MESS??.DAT file with outgoing messages.                               */
/* First word is #messages in file                                       */
/* Then an ARRAY[50] OF MESSAGE_ENTRY (see below)                        */
/* Then the actual messages, add an offset (+13) to each letter          */
/* Note: no more than 50 messages can be transmitted                     */
/*************************************************************************/

typedef struct
{ UB2  iTo;                 // Destination of message
  UB4  lOffset;             // Start+1 in file
  UB2  wLength;             // Length of message in bytes
  XString sMessage;
} OUTMESS;

/*************************************************************************/
/* VCR??.DAT. File for the Visual Combat Recorder.                       */
/* The first UB2  of the file is #battles.                               */
/* ( Thanks Alistair )                                                   */
/*************************************************************************/

/* Note: Not tested */

typedef struct
{
   UB2  wMass;              // Mass of ships
   XString sName;           // Name of ship [20]
   UB2  wDamage;            // Initial damage state of ship
   UB2  Crew;               // Initial crew on ship
   UB2  wID;                // ID of ship
   UB2  wRace;              // Owner of ship
   UB2  wBitmap;            // Index in RESOURCE.PLN
   UB2  wBeamTech;          // Tech. level of beam weapons
   UB2  wBeams;             // Number of beam weapons
   UB2  wBays;              // Number of fighter bays
   UB2  wTorpTech;          // Tech. level of torp. launchers
   UB2  wTorpFighters;      // Initial number of torps. or fighter on ship
   UB2  wTorps;             // Number of torp. lauchers
   UB2  wShield;            // Initial shields of both ships.
} COMBATANT;

typedef struct
{  UB4  RandomSeed;
   UB2  wPlanetTemp;        /* Temperature of planet */
   UB2  IsPlanet;           /* Tells whether right side is a planet or base */
   COMBATANT combatant[3];  // combatants
} VCR_DATA;

/**************************************************************************/
/* BDATA??.DAT  First UB2  #bases, followed by a list of bases.           */
/**************************************************************************/

typedef struct
{ UB2  wID;
  UB2  wOwner;
  UB2  wDefence;       // Defence of starbase
  UB2  wDamage;        // Damage in %

  /* Tech levels */
  UB2  wTechEngine;
  UB2  wTechHull;
  UB2  wTechBeam;
  UB2  wTechTorp;

  /* Items on starbase */
  UB2  wEngines[10];    // #engines on starbase index 0=stardrive 1    0x12
  UB2  wHulls[21];      // #Hulls, index = index in truehull           0x24
  UB2  wBeams[11];      // #beam weapons on starbase index 0=Laser     0x4C
  UB2  wTLaunchers[11]; // #torp lauchers on base, 0=Mark 1            0x60
  UB2  wTorps[11];      // #tops on starbase index 0=Mark 1            0x74

  UB2  wFighters;       // #fighters on base 0x88
  UB2  wFixID;          // ID of ship being fixed or recycled
  UB2  wFixShip;        // 0=do nothing, 1=fix ship, 2=recycle ship
  UB2  wMission;        // 1 = Refuel, 2 = Max defence, ...

  /* Ship being build */
  UB2  wShip;           // Ship being build (index in Hullspel?)
  UB2  wEngineLevel;    // Engine level of ship being build
  UB2  wBeamLevel;      // Beam level of ship being build
  UB2  wBeamOnShip;     // Number of beams on ship
  UB2  wTorpLevel;      // Torp. laucher tech level
  UB2  wTorpOnShip;     // Number of torp. laucher on ship

  UB2  wFighterOnShip;  // ignore, equals 0

} BASEDATA;

typedef struct
{ Coord pos;
  UB2 wRace;
  UB4 lUnits;
  UB2 wWeb;
} MINEDATA;

typedef struct
{ UB2 wRecycleRate; // recycle rate of colonising ships %
  UB2 wRandomMeteorRate; // odds of a large meteor impact %
  UB2 wAllowMinefields; // minefields 1/0
  UB2 wAllowAlchemy; // alchemy ships 1/0
  UB2 wDeleteOldMessages; // delete old messages 1/0
  UB2 wDisablePasswords; // disable all passwords 1/0
  UB2 wGroundKillFactor[12]; // 11 x ground combat attack ratio (12)
  UB2 wGroundDefenseFactor[12]; // 11 x ground combat defence ratio (12)
  UB2 wFreeFighters[12]; // 11 x free starbase fighters (12)
  UB2 wRaceMiningRate[12]; // 11 x mineral mining rate % (12)
  UB2 wRaceTaxRate[12]; // 11 x tax collection rate %
  UB2 wRebelsBuildFighters; // rebels build fighters in space
  UB2 wColoniesBuildFighters; // colonies build fighters in space
  UB2 wRobotsBuildFighters; // robots build fighters in space
  UB2 wCloakFailureRate; // cloak failure odds
  UB2 wRobCloakedShips; // rob cloaked ships 1/0
  UB2 wScanRange; // ship visible range (starcharts)
  UB2 wDarkSenseRange; // Empire dark sense range
  UB2 wAllowHiss; // lizard hiss 1/0
  UB2 wAllowRebelGroundAttack; // rebel ground attack 1/0
  UB2 wAllowSuperRefit; // fed super refit 1/0
  UB2 wAllowWebMines; // web minefields 1/0
  UB2 wCloakFuelBurn; // fuel used to cloak
  UB2 wSensorRange; // sensor mission range
  UB2 wAllowNewNatives; // new natives may appear 1/0
  UB2 wAllowNUKCode; // NUK code
  UB2 wBorgAssimilationRate; // cyborg assimilation rate
  UB2 wWebMineDecayRate; // web mine decay rate
  UB2 wMineDecayRate; // minefield decay rate
  UB2 wMaximumMinefieldRadius; // Maximum mine field radius
  UB2 wTransuraniumDecayRate; // new mineral formation rate
  UB2 wStructureDecayPerTurn; // planetary structure decay rate
  UB2 wAllowEatingSupplies; // overpopulation eats supplies 1/0
  UB2 wAllowNoFuelMovement; // ships w/o fuel can move 1/0
  UB2 wMineHitOdds; // Odds of hitting a mine per L/Y
  UB2 wWebMineHitOdds; // Odds of hitting a web per L/Y
  UB2 wMineScanRange; // Mine detect range
  UB2 wAllowMinesDestroyMines; // Mines destroy enemy mines 1/0
  UB2 wAllowEngineShieldBonus; // Engine tech boosts shield power 1/0
  UB2 wEngineShieldBonusRate; // Engine tech boosts shield power %
  UB2 wColonialFighterSweepRate; // colonial fighter mine sweep rate
  UB2 wAllowColoniesSweepWebs; // colonial fighter mine sweep webs 1/0
  UB2 wMineSweepRate; // Mine field sweep rate
  UB2 wWebMineSweepRate; // Web mine field sweep rate
  UB2 wHissEffectRate; // hiss mission effectiveness
  UB2 wRobFailureOdds; // rob mission failure %
  UB2 wPlanetsAttackRebels; // attack rebels on GA 1/0
  UB2 wPlanetsAttackKlingons; // attack fascist on pillage 1/0
  UB2 wMineSweepRange; // Mine field sweep range  (LY's)
  UB2 wWebMineSweepRange; // Web mine field sweep range (LY's)
  UB2 wAllowScienceMission; // Terraformers 1/0
  UB2 wMineHitOddsWhenCloakedX10; // Cloaked ship will hit mine odds (/1000)
  UB2 wDamageLevelForCloakFail; // Amount of damage that prevents cloak
  UB2 wAllowFedCombatBonus; // fed crew bonus
  UB2 wMeteorShowerOdds; // Odds of small meteor impact on all
  UB4 wMeteorShowerOreMinFuel; //  Minimum Neutronium
  UB4 wMeteorShowerOreMinDur; //  Minimum Duranium
  UB4 wMeteorShowerOreMinTri; //  Minimum Tritanium
  UB4 wMeteorShowerOreMinMol; //  Minimum Molybdenum
  UB4 wMeteorShowerOreMaxFuel; //  Maximum Neutronium
  UB4 wMeteorShowerOreMaxDur; //  Maximum Duranium
  UB4 wMeteorShowerOreMaxTri; //  Maximum Tritanium
  UB4 wMeteorShowerOreMaxMoly; //  Maximum Molybdenum
  UB2 wLargeMeteorsImpacting; // Number of LARGE meteors impacting
  UB4 wLargeMeteorOreMinFuel; //  Minimum Neutronium
  UB4 wLargeMeteorOreMinDur; //  Minimum Duranium
  UB4 wLargeMeteorOreMinTri; //  Minimum Tritanium
  UB4 wLargeMeteorOreMinMoly; //  Minimum Molybdenum
  UB4 wLargeMeteorOreMaxFuel; //  Maximum Neutronium
  UB4 wLargeMeteorOreMaxDur; //  Maximum Duranium
  UB4 wLargeMeteorOreMaxTri; //  Maximum Tritanium
  UB4 wLargeMeteorOreMaxMoly; //  Maximum Molybdenum
  UB2 wAllowMeteorMessages; // Send Meteor Messages
  UB2 wAllowOneEngineTowing; // Ships with one engine can tow 1/0
  UB2 wAllowHyperWarps; // Hyperdrive ships 1/0
  UB2 wClimateDeathRate; // Climate death rate %
  UB2 wAllowGravityWells; // Planets have gravity wells 1/0
  UB2 wCrystalsPreferDeserts; // Crystal people like desert worlds 1/0
  UB2 wAllowMinesDestroyWebs; // Normal mines destroy web mines 1/0
  UB2 wClimateLimitsPopulation; // Climate limits population 1/0
  UB4 wMaxPlanetaryIncome; // Maximum income per planet
  UB2 wIonStormActivity; // Ion Storm activity
  UB2 wAllowChunneling; // Firecloud Warp Chunnel 1/0
  UB2 wAllowDeluxeSuperSpy; // Advanced Birdmen Super Spi 1/0
  UB2 wIonStormsHideMines; // Ion storms hide minefields 1/0
  UB2 wAllowGloryDevice; // Glory device (D19, Saber) 1/0
  UB2 wAllowAntiCloakShips; // Loki Class Anti-cloaker function 1/0
  UB2 wAllowGamblingShips; // Lady Royale Gambling Ship 1/0
  UB2 wAllowCloakedShipsAttack; // Cloaked ships can attack 1/0
  UB2 wAllowCloning; // Ship Cloning 1/0
  UB2 wAllowBoardingParties; // Crystal/Privateer Boarding Party 1/0
  UB2 wAllowImperialAssault; // Star Destroyer Imperial Assault 1/0
  UB2 wRamScoopFuelPerLY; // Cobol class free fuel per LY
  UB2 wAllowAdvancedRefinery; // Aries can convert minerals to fuel 1/0
  UB2 wAllowBioscanners; // Bioscanners 1/0
  UB2 wHullTechNotSlowedByMines; // Hull Tech not slowed by mine hits
} HCONFIG;

typedef struct
{ UB2 wScenarioActive; // is a scenario active?
  UB2 wMasterScenario;
  UB2 wHomeworld[12]; // used in scenario 6, 7
  UB2 wTantalusTurnsOwned[12]; // used in scenario 4
  UB2 wTantalusPlanet; // used in scenario 4
  UB2 wTantalusTurnsNeeded; // used in scenario 4
  UB2 wOldEmpire; // race #, used in scenario 6
  UB2 wScenario; // 1 Colonist/Native Elections
                 // 2 Ship Tonnage
                 // 3 Elections and Ship Tonnage
                 // 4 The Tantalus Machine
                 // 5 Moly the Spice of life
                 // 6 Ashes of the Old Empire
                 // 7 Invasion
  SB4 lScore[12]; // total; used in scenarios 1-3, 5, 7
  UB4 lScoreNeeded; // to win; used in scenarios 1-3, 5, 7
  UB2 wWinner; // do we have a winner?
} RCONFIG;

/* gen.hst */

typedef struct
{ UB2 wPlaying[12];
  XString sPassword[12];
  UB2 wLastNormalBuild;
} GENHOST;

/* speed is not stored here */

typedef struct
{ Coord pos;
  UB2 wRadius;
  UB2 wVoltage;
  UB2 wHeading;
  UB2 wGrowShrink;
} ION_STORM;

typedef struct
{  XString sName; // [20]
} ION_NAME;

typedef struct
{  UB2 wExperience;
} ION_SHIPEX;

typedef struct
{ UB2 PBP;
} PBPDATA;

typedef struct
{ UB2 wMarker;    // 0 none, 1-15 colour
  XString sNm;    // Name on map [20]
  XString Info1;  // High-res info [20]
  XString Info2;  // High-res info [20]
  Coord pos;
  UB2 wWarp;      // Speed (0-15)
  UB2 wHeading;   // as usual
  UB2 wRangeP;    // range at which visible from planet
  UB2 wRangeS;    // range at which visible from ship
  UB2 wEdge;      // Radius (LY)
  UB2 wParent;    // Program ID #
} UFO_DATA;

extern BASEDATA *read_all_basedata();
extern BASEDATA *read_basedata(int bnum);
extern BEAMSPEC *read_all_beamspec();
extern BEAMSPEC *read_beamspec(int bnum);
extern bool del_outmess(int mnum);
extern bool isally(int rnumfrom,int rnumto);
extern bool make_checksum();
extern bool set_game_dir(const char *gdir);
extern bool write_basedata(int bnum, BASEDATA *str);
extern bool write_beamspec(int bnum, BEAMSPEC *str);
extern bool write_engspec(int e_num, ENGSPEC *str);
extern bool write_genhost(GENHOST *str);
extern bool write_hconfig(HCONFIG *str);
extern bool write_hullspec(int hnum, HULLSPEC *str);
extern bool write_ion_name(int inum, ION_NAME *str);
extern bool write_ion_shipex(int snum, ION_SHIPEX *str);
extern bool write_ion_storm(int inum, ION_STORM *str);
extern bool write_minedata(int mnum, MINEDATA *str);
extern bool write_outmess(int mnum, OUTMESS *str);
extern bool write_pbpdata(int rnum, PBPDATA *str);
extern bool write_planetdata(int pnum, PLANETDATA *str);
extern bool write_planet_coords(int pnum, PLANET_COORDS *str);
extern bool write_planet_name(int pnum, PLANET_NAME *str);
extern bool write_racenm(int rnum, RACENM *str);
extern bool write_rconfig(RCONFIG *str);
extern bool write_shipdata(int snum, SHIPDATA *str);
extern bool write_shipxy(int snum, SHIPXY *str);
extern bool write_torpspec(int tnum, TORPSPEC *str);
extern bool write_truehull(int hnum, int playernum, TRUEHULL *str);
extern bool write_ufo_data(int inum, UFO_DATA *str);
extern bool write_vcr_data(int vnum, VCR_DATA *str);
extern ENGSPEC *read_all_engspec();
extern ENGSPEC *read_engspec(int e_num);
extern FILE *mkopen(const char *filename, bool trylocal, char mode);
extern GENHOST *read_genhost();
extern HCONFIG *read_hconfig();
extern HULLSPEC *read_all_hullspec();
extern HULLSPEC *read_hullspec(int hnum);
extern int num_outmess();
extern int num_vcrs();
extern ION_NAME *read_all_ion_name();
extern ION_NAME *read_ion_name(int inum);
extern ION_SHIPEX *read_all_ion_shipex();
extern ION_SHIPEX *read_ion_shipex(int snum);
extern ION_STORM *read_all_ion_storm();
extern ION_STORM *read_ion_storm(int inum);
extern MINEDATA *read_all_minedata();
extern MINEDATA *read_minedata(int mnum);
extern OUTMESS *read_all_outmess();
extern OUTMESS *read_outmess(int mnum);
extern PBPDATA *read_all_pbpdata();
extern PBPDATA *read_pbpdata(int rnum);
extern PLANETDATA *read_all_planetdata();
extern PLANETDATA *read_planetdata(int pnum);
extern PLANET_COORDS *read_all_planet_coords();
extern PLANET_COORDS *read_planet_coords(int pnum);
extern PLANET_NAME *read_all_planet_name();
extern PLANET_NAME *read_planet_name(int pnum);
extern RACENM *read_all_racenm();
extern RACENM *read_racenm(int rnum);
extern RCONFIG *read_rconfig();
extern SHIPDATA *read_all_shipdata();
extern SHIPDATA *read_shipdata(int snum);
extern SHIPXY *read_all_shipxy();
extern SHIPXY *read_shipxy(int snum);
extern TORPSPEC *read_all_torpspec();
extern TORPSPEC *read_torpspec(int tnum);
extern TRUEHULL *read_all_truehull(int pn);
extern TRUEHULL *read_truehull(int hnum, int playernum);
extern UFO_DATA *read_all_ufo_data();
extern UFO_DATA *read_ufo_data(int inum);
extern VCR_DATA *read_all_vcr_data();
extern VCR_DATA *read_vcr_data(int vnum);
extern void setally(int rnumfrom,int rnumto,bool allying);
extern XString rcode();
extern bool extraplan;
extern bool extraship;

#endif

