// -*- C++ -*-

#ifndef _XSTRING_H
#define _XSTRING_H

#include <stddef.h>
#include <stdarg.h>

/*
  Portable string class
  Copyright Keristor Systems and Chris Croughton 1997
  Internet: chris@keris.demon.co.uk
  
  This software is free software; you can redistribute it and/or modify
  it freely under the following conditions:

  1) The copyright of this software belongs to the above copyright holders,
     and all versions of the software must carry (at least in source form)
     the copyright notice and these conditions.

  2) Permission is granted for any non-commercial use for any purpose; if
     the software is distributed in modified form then in addition to the
     previous restriction it must also bear a notice stating that it has
     been modified and by whom.

  3) Any modifications are your responsibility; if you wish to pass them on
     to the authors then we reserve the right to include them or not, as we
     decide, in future versions of the software.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

// NOTE:
// The interfaces here are subject to change, this software is still
// under development.  In particular, the '+' operators for concatenation
// of strings are being replaced by equivalent '|' operators; the '+'
// operators will eventually have a different effect.

/*
 * $RCSfile: xstring.h,v $
 *
 * $Date: 1998/01/14 12:41:07 $
 * $Revision: 1.11 $  
 *
 * $Log: xstring.h,v $
 * Revision 1.11  1998/01/14  12:41:07  crough45
 * Changed header.
 *
 * Revision 1.10  1997/11/10  13:35:34  crough45
 * Convert return type of Find* functions to long rather than
 * int, to allow maximum string length plus -1 for not found.
 *
 * Revision 1.9  1997/11/10  13:20:12  crough45
 * Usage condition changes
 * Add constructors taking integers (to be converted to ASCII)
 * Add MixedCase conversion functions
 * Add PadRight, PadLeft and Centre functions
 * Add start character number to Find* functions
 *
 * Revision 1.8  1997/07/21  11:43:42  crough45
 * Added operators | etc. to replace + for concatenation;
 * added concatenation of integer values to append the
 * values in decimal
 *
 * Revision 1.7  1997/03/15  14:51:06  crough45
 * Added operator = for const void *.
 *
 * Revision 1.6  1997/03/06  16:17:26  crough45
 * Added operator= for void pointer
 *
 * Revision 1.2  1997/02/18  12:59:35  crough45
 * Changed header
 *
 * Revision 1.1  1997/02/18  12:42:51  crough45
 * Initial revision
 *
*/

class XString
{
public:
// Constructors
   XString();
   XString(const XString& string);
   XString(char ch, int nRepeat = 1);
   XString(const char * lpsz);
   XString(const char * lpch, int nLength);
   XString(int n);
   XString(unsigned int n);
   XString(long n);
   XString(unsigned long n);
   
// Attributes & Operations
   // as an array of characters
   int Len() const;
   char IsEmpty() const;
   void Empty();                      
   
   char GetAt(int nIndex) const;      
   char operator[](int nIndex) const; 
   void SetAt(int nIndex, char ch);
   operator const char *() const {return (m_string ? m_string : "");}
   operator bool () const {return (m_string ? *m_string != 0 : false);}
   
   // overloaded assignment
   const XString& operator=(const XString& stringSrc);
   const XString& operator=(char ch);
   const XString& operator=(const char * lpsz);
   const XString& operator=(const unsigned char * psz);
   const XString& operator=(const void * p) 
   { operator=((const char *)p); return *this; }
   const XString& operator=(int n) 
   { if (!n) operator=((const char *)0); return *this; }
   
   // string concatenation
   const XString& operator+=(const XString& string);
   const XString& operator+=(char ch);
   const XString& operator+=(const char * lpsz);
   
   friend XString operator+(const XString& string1,
                            const XString& string2);
   friend XString operator+(const XString& string, char ch);
   friend XString operator+(char ch, const XString& string);
   friend XString operator+(const XString& string, const char * lpsz);
   friend XString operator+(const char * lpsz, const XString& string);
   
   const XString& operator|=(const XString& string)
   {return operator+=(string); }
   const XString& operator|=(char ch)
   { return operator+=(ch); }
   const XString& operator|=(const char * lpsz)
   { return operator+=(lpsz); }
   
   friend XString operator|(const XString& string1,
                            const XString& string2)
   { return operator+(string1,string2); }
   friend XString operator|(const XString& string, char ch)
   { return operator+(string, ch); }
   friend XString operator|(char ch, const XString& string)
   { return operator+(ch, string); }
   friend XString operator|(const XString& string, const char * lpsz)
   { return operator+(string, lpsz); }
   friend XString operator|(const char * lpsz, const XString& string)
   { return operator+(lpsz, string); }

   friend XString operator|(const XString& s, long n);
   friend XString operator|(const XString& s, unsigned long n);
   friend XString operator|(const XString& s, int n)
   { return operator|(s, (long)n); }
   friend XString operator|(const XString& s, unsigned int n)
   { return operator|(s, (unsigned long)n); }
   friend XString operator|(const XString& s, short n)
   { return operator|(s, (long)n); }
   friend XString operator|(const XString& s, unsigned short n)
   { return operator|(s, (unsigned long)n); }

   friend XString operator+(const XString& s)
   {return s.Upper();}
   friend XString operator-(const XString& s)
   {return s.Lower();}
   friend XString operator~(const XString& s)
   {return s.OtherCase();}
   friend XString operator*(const XString& s)
   {return s.MixedCase();}
   
   // string comparison
   int Compare(const char * lpsz) const;         // straight character
   int CompareNoCase(const char * lpsz) const;   // ignore case
   int Collate(const char * lpsz) const;         // NLS aware
   
   // simple sub-string extraction
   XString Mid(int nFirst, int nCount) const;
   XString Mid(int nFirst) const;
   XString Left(int nCount) const;
   XString Right(int nCount) const;
   
   const XString& operator<<=(const int nChars);

   XString SpanIncluding(const char * lpszCharSet) const;
   XString SpanExcluding(const char * lpszCharSet) const;
   
   // upper/lower/reverse/capitalise conversion
   void ToUpper();
   void ToLower();
   void ToOtherCase();
   void ToMixedCase();
   
   XString Upper() const;
   XString Lower() const;
   XString OtherCase() const;
   XString MixedCase() const;

   friend XString Upper(const char *s)
   { return ((XString)s).Upper(); }
   friend XString Lower(const char *s)
   { return ((XString)s).Lower(); }
   friend XString OtherCase(const char *s)
   { return ((XString)s).OtherCase(); }
   friend XString MixedCase(const char *s)
   { return ((XString)s).MixedCase(); }
   
   // trimming whitespace (either side)
   void TrimRight();
   void TrimLeft();
   void Trim();

   // adding whitespace (either side)
   friend XString PadRight(const char *s, int len);
   friend XString PadLeft(const char *s, int len);
   friend XString Centre(const char *s, int len);
   
   // searching (return starting index, or -1 if not found)
   // look for a single character match
   long Find(char ch, size_t first=0) const;             // like "C" strchr
   long FindLast(char ch, size_t first=0) const;
   long FindOneOf(const char * lpszCharSet, size_t first=0) const;
   
   // look for a specific sub-string
   long Find(const char * lpszSub, size_t first=0) const; // like "C" strstr
   
   // simple formatting
   void Format(const char * lpszFormat, ...);
   void vFormat(const char * lpszFormat, va_list ap);
   friend XString xStrFormat(const char * lpszFormat, ...)
   {
     XString s;
     va_list ap;
     va_start(ap, lpszFormat);
     s.vFormat(lpszFormat, ap);
     va_end(ap);
     return s;
   }
   friend XString vxStrFormat(const char * lpszFormat, va_list ap)
   {
     XString s;
     s.vFormat(lpszFormat, ap);
     return s;
   }
   
   // Access to string implementation buffer as "C" character array
   const char * GetBuffer(size_t nMinBufLength);
   void ReleaseBuffer(size_t nNewLength = -1);
   const char * GetBufferSetLength(size_t nNewLength);
   void FreeExtra();
   
   // Use LockBuffer/UnlockBuffer to turn refcounting off
   const char * LockBuffer();
   void UnlockBuffer();

// Implementation
public:
   ~XString();
   int GetAllocLength() const;

protected:
   char *m_string;   // pointer to null terminated string data
};

char operator==(const XString& s1, const XString& s2);
char operator==(const XString& s1, const char * s2);
char operator==(const char * s1, const XString& s2);
char operator!=(const XString& s1, const XString& s2);
char operator!=(const XString& s1, const char * s2);
char operator!=(const char * s1, const XString& s2);
char operator<(const XString& s1, const XString& s2);
char operator<(const XString& s1, const char * s2);
char operator<(const char * s1, const XString& s2);
char operator>(const XString& s1, const XString& s2);
char operator>(const XString& s1, const char * s2);
char operator>(const char * s1, const XString& s2);
char operator<=(const XString& s1, const XString& s2);
char operator<=(const XString& s1, const char * s2);
char operator<=(const char * s1, const XString& s2);
char operator>=(const XString& s1, const XString& s2);
char operator>=(const XString& s1, const char * s2);
char operator>=(const char * s1, const XString& s2);
char operator*=(const XString& s1, const XString& s2);
char operator*=(const XString& s1, const char * s2);
char operator*=(const char * s1, const XString& s2);
char operator/=(const XString& s1, const XString& s2);
char operator/=(const XString& s1, const char * s2);
char operator/=(const char * s1, const XString& s2);

#endif
